'''OpenGL extension VERSION.GL_4_0

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_4_0 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_4_0.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL import _types
from OpenGL.raw.GL.VERSION.GL_4_0 import *
from OpenGL.raw.GL.VERSION.GL_4_0 import _EXTENSION_NAME

def glInitGl40VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

### END AUTOGENERATED SECTION
from OpenGL.GL.ARB.texture_query_lod import *
from OpenGL.GL.ARB.draw_indirect import *
from OpenGL.GL.ARB.gpu_shader5 import *
from OpenGL.GL.ARB.gpu_shader_fp64 import *
from OpenGL.GL.ARB.shader_subroutine import *
from OpenGL.GL.ARB.tessellation_shader import *
from OpenGL.GL.ARB.texture_buffer_object_rgb32 import *
from OpenGL.GL.ARB.texture_cube_map_array import *
from OpenGL.GL.ARB.texture_gather import *
from OpenGL.GL.ARB.transform_feedback2 import *
from OpenGL.GL.ARB.transform_feedback3 import *
