"""glGet* auto-generation of output arrays (DO NOT EDIT, AUTOGENERATED)"""
_glget_size_mapping = _m = {}
# _m[0x92C1] = TODO # GL_ATOMIC_COUNTER_BUFFER_BINDING
# _m[0x843A] = TODO # GL_BINORMAL_ARRAY_EXT
# _m[0x9285] = TODO # GL_BLEND_ADVANCED_COHERENT_NV
# _m[0x9281] = TODO # GL_BLEND_OVERLAP_NV
# _m[0x9280] = TODO # GL_BLEND_PREMULTIPLIED_SRC_NV
# _m[0x8076] = TODO # GL_COLOR_ARRAY_EXT
# _m[0x8A0F] = TODO # GL_COLOR_FLOAT_APPLE
# _m[0x8E20] = TODO # GL_COLOR_SAMPLES_NV
# _m[0x90FB] = TODO # GL_COMPUTE_PROGRAM_NV
# _m[0x00000002] = TODO # GL_CONTEXT_FLAG_DEBUG_BIT
# _m[0x00000004] = TODO # GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB
# _m[0x81AA] = TODO # GL_CULL_VERTEX_EXT
# _m[0x81AB] = TODO # GL_CULL_VERTEX_EYE_POSITION_EXT
# _m[103050] = TODO # GL_CULL_VERTEX_IBM
# _m[0x81AC] = TODO # GL_CULL_VERTEX_OBJECT_POSITION_EXT
# _m[0x8843] = TODO # GL_CURRENT_PALETTE_MATRIX_ARB
# _m[0x826D] = TODO # GL_DEBUG_GROUP_STACK_DEPTH
# _m[0x9145] = TODO # GL_DEBUG_LOGGED_MESSAGES
# _m[0x9145] = TODO # GL_DEBUG_LOGGED_MESSAGES_AMD
# _m[0x9145] = TODO # GL_DEBUG_LOGGED_MESSAGES_ARB
# _m[0x8243] = TODO # GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH
# _m[0x8243] = TODO # GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH_ARB
# _m[0x8DAF] = TODO # GL_DEPTH_BUFFER_FLOAT_MODE_NV
# _m[0x864F] = TODO # GL_DEPTH_CLAMP
# _m[0x901F] = TODO # GL_DEPTH_CLAMP_FAR_AMD
# _m[0x901E] = TODO # GL_DEPTH_CLAMP_NEAR_AMD
# _m[0x90EF] = TODO # GL_DISPATCH_INDIRECT_BUFFER_BINDING
# _m[0x8CA6] = TODO # GL_DRAW_FRAMEBUFFER_BINDING
# _m[0x8F43] = TODO # GL_DRAW_INDIRECT_BUFFER_BINDING
# _m[0x8E5D] = TODO # GL_FRAGMENT_INTERPOLATION_OFFSET_BITS
# _m[0x8E5D] = TODO # GL_FRAGMENT_PROGRAM_INTERPOLATION_OFFSET_BITS_NV
# _m[0x8DA4] = TODO # GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV
# _m[0x8DB9] = TODO # GL_FRAMEBUFFER_SRGB
# _m[0x8DBA] = TODO # GL_FRAMEBUFFER_SRGB_CAPABLE_EXT
# _m[0x8DB9] = TODO # GL_FRAMEBUFFER_SRGB_EXT
# _m[0x8C26] = TODO # GL_GEOMETRY_PROGRAM_NV
# _m[0x8DA3] = TODO # GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV
# _m[0x8B9B] = TODO # GL_IMPLEMENTATION_COLOR_READ_FORMAT
# _m[0x8B9A] = TODO # GL_IMPLEMENTATION_COLOR_READ_TYPE
# _m[0x825E] = TODO # GL_LAYER_PROVOKING_VERTEX
# _m[0x8252] = TODO # GL_LOSE_CONTEXT_ON_RESET_ARB
# _m[0x92DC] = TODO # GL_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS
# _m[0x92D8] = TODO # GL_MAX_ATOMIC_COUNTER_BUFFER_SIZE
# _m[0x92D7] = TODO # GL_MAX_COMBINED_ATOMIC_COUNTERS
# _m[0x92D1] = TODO # GL_MAX_COMBINED_ATOMIC_COUNTER_BUFFERS
# _m[0x8266] = TODO # GL_MAX_COMBINED_COMPUTE_UNIFORM_COMPONENTS
# _m[0x90CF] = TODO # GL_MAX_COMBINED_IMAGE_UNIFORMS
# _m[0x8F39] = TODO # GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS
# _m[0x8F39] = TODO # GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS_EXT
# _m[0x90DC] = TODO # GL_MAX_COMBINED_SHADER_STORAGE_BLOCKS
# _m[0x8E1E] = TODO # GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS
# _m[0x8E1F] = TODO # GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS
# _m[0x8265] = TODO # GL_MAX_COMPUTE_ATOMIC_COUNTERS
# _m[0x8264] = TODO # GL_MAX_COMPUTE_ATOMIC_COUNTER_BUFFERS
# _m[0x91BD] = TODO # GL_MAX_COMPUTE_IMAGE_UNIFORMS
# _m[0x90DB] = TODO # GL_MAX_COMPUTE_SHADER_STORAGE_BLOCKS
# _m[0x8262] = TODO # GL_MAX_COMPUTE_SHARED_MEMORY_SIZE
# _m[0x91BC] = TODO # GL_MAX_COMPUTE_TEXTURE_IMAGE_UNITS
# _m[0x91BB] = TODO # GL_MAX_COMPUTE_UNIFORM_BLOCKS
# _m[0x8263] = TODO # GL_MAX_COMPUTE_UNIFORM_COMPONENTS
# _m[0x9344] = TODO # GL_MAX_COMPUTE_VARIABLE_GROUP_INVOCATIONS_ARB
# _m[0x90EB] = TODO # GL_MAX_COMPUTE_WORK_GROUP_INVOCATIONS
# _m[0x826C] = TODO # GL_MAX_DEBUG_GROUP_STACK_DEPTH
# _m[0x9144] = TODO # GL_MAX_DEBUG_LOGGED_MESSAGES
# _m[0x9144] = TODO # GL_MAX_DEBUG_LOGGED_MESSAGES_AMD
# _m[0x9144] = TODO # GL_MAX_DEBUG_LOGGED_MESSAGES_ARB
# _m[0x9143] = TODO # GL_MAX_DEBUG_MESSAGE_LENGTH
# _m[0x9143] = TODO # GL_MAX_DEBUG_MESSAGE_LENGTH_AMD
# _m[0x9143] = TODO # GL_MAX_DEBUG_MESSAGE_LENGTH_ARB
# _m[0x90D1] = TODO # GL_MAX_DEEP_3D_TEXTURE_DEPTH_NV
# _m[0x90D0] = TODO # GL_MAX_DEEP_3D_TEXTURE_WIDTH_HEIGHT_NV
# _m[0x88FC] = TODO # GL_MAX_DUAL_SOURCE_DRAW_BUFFERS
# _m[0x92D6] = TODO # GL_MAX_FRAGMENT_ATOMIC_COUNTERS
# _m[0x92D0] = TODO # GL_MAX_FRAGMENT_ATOMIC_COUNTER_BUFFERS
# _m[0x90CE] = TODO # GL_MAX_FRAGMENT_IMAGE_UNIFORMS
# _m[0x8E5C] = TODO # GL_MAX_FRAGMENT_INTERPOLATION_OFFSET
# _m[0x8E5C] = TODO # GL_MAX_FRAGMENT_INTERPOLATION_OFFSET_NV
# _m[0x90DA] = TODO # GL_MAX_FRAGMENT_SHADER_STORAGE_BLOCKS
# _m[0x8DFD] = TODO # GL_MAX_FRAGMENT_UNIFORM_VECTORS
# _m[0x9316] = TODO # GL_MAX_FRAMEBUFFER_HEIGHT
# _m[0x9317] = TODO # GL_MAX_FRAMEBUFFER_LAYERS
# _m[0x9318] = TODO # GL_MAX_FRAMEBUFFER_SAMPLES
# _m[0x9315] = TODO # GL_MAX_FRAMEBUFFER_WIDTH
# _m[0x92D5] = TODO # GL_MAX_GEOMETRY_ATOMIC_COUNTERS
# _m[0x92CF] = TODO # GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS
# _m[0x90CD] = TODO # GL_MAX_GEOMETRY_IMAGE_UNIFORMS
# _m[0x8E5A] = TODO # GL_MAX_GEOMETRY_PROGRAM_INVOCATIONS_NV
# _m[0x8E5A] = TODO # GL_MAX_GEOMETRY_SHADER_INVOCATIONS
# _m[0x90D7] = TODO # GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS
# _m[0x906D] = TODO # GL_MAX_IMAGE_SAMPLES
# _m[0x906D] = TODO # GL_MAX_IMAGE_SAMPLES_EXT
# _m[0x8F38] = TODO # GL_MAX_IMAGE_UNITS
# _m[0x8F38] = TODO # GL_MAX_IMAGE_UNITS_EXT
# _m[0x82E8] = TODO # GL_MAX_LABEL_LENGTH
# _m[0x8E7D] = TODO # GL_MAX_PATCH_VERTICES
# _m[0x8905] = TODO # GL_MAX_PROGRAM_TEXEL_OFFSET_EXT
# _m[0x8F9F] = TODO # GL_MAX_PROGRAM_TEXTURE_GATHER_COMPONENTS_ARB
# _m[0x8E5F] = TODO # GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_ARB
# _m[0x8E5F] = TODO # GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_NV
# _m[0x90DE] = TODO # GL_MAX_SHADER_STORAGE_BLOCK_SIZE
# _m[0x90DD] = TODO # GL_MAX_SHADER_STORAGE_BUFFER_BINDINGS
# _m[0x9199] = TODO # GL_MAX_SPARSE_3D_TEXTURE_SIZE_AMD
# _m[0x9199] = TODO # GL_MAX_SPARSE_3D_TEXTURE_SIZE_ARB
# _m[0x919A] = TODO # GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS
# _m[0x919A] = TODO # GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS_ARB
# _m[0x9198] = TODO # GL_MAX_SPARSE_TEXTURE_SIZE_AMD
# _m[0x9198] = TODO # GL_MAX_SPARSE_TEXTURE_SIZE_ARB
# _m[0x8DE7] = TODO # GL_MAX_SUBROUTINES
# _m[0x8DE8] = TODO # GL_MAX_SUBROUTINE_UNIFORM_LOCATIONS
# _m[0x92D3] = TODO # GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS
# _m[0x92CD] = TODO # GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS
# _m[0x90CB] = TODO # GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS
# _m[0x886C] = TODO # GL_MAX_TESS_CONTROL_INPUT_COMPONENTS
# _m[0x8E83] = TODO # GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS
# _m[0x90D8] = TODO # GL_MAX_TESS_CONTROL_SHADER_STORAGE_BLOCKS
# _m[0x8E81] = TODO # GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS
# _m[0x8E85] = TODO # GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS
# _m[0x8E89] = TODO # GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS
# _m[0x8E7F] = TODO # GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS
# _m[0x92D4] = TODO # GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS
# _m[0x92CE] = TODO # GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS
# _m[0x90CC] = TODO # GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS
# _m[0x886D] = TODO # GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS
# _m[0x8E86] = TODO # GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS
# _m[0x90D9] = TODO # GL_MAX_TESS_EVALUATION_SHADER_STORAGE_BLOCKS
# _m[0x8E82] = TODO # GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS
# _m[0x8E8A] = TODO # GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS
# _m[0x8E80] = TODO # GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS
# _m[0x8E7E] = TODO # GL_MAX_TESS_GEN_LEVEL
# _m[0x8E84] = TODO # GL_MAX_TESS_PATCH_COMPONENTS
# _m[0x8E70] = TODO # GL_MAX_TRANSFORM_FEEDBACK_BUFFERS
# _m[0x826E] = TODO # GL_MAX_UNIFORM_LOCATIONS
# _m[0x8DFC] = TODO # GL_MAX_VARYING_VECTORS
# _m[0x92D2] = TODO # GL_MAX_VERTEX_ATOMIC_COUNTERS
# _m[0x92CC] = TODO # GL_MAX_VERTEX_ATOMIC_COUNTER_BUFFERS
# _m[0x82DA] = TODO # GL_MAX_VERTEX_ATTRIB_BINDINGS
# _m[0x82D9] = TODO # GL_MAX_VERTEX_ATTRIB_RELATIVE_OFFSET
# _m[0x90CA] = TODO # GL_MAX_VERTEX_IMAGE_UNIFORMS
# _m[0x90D6] = TODO # GL_MAX_VERTEX_SHADER_STORAGE_BLOCKS
# _m[0x8E71] = TODO # GL_MAX_VERTEX_STREAMS
# _m[0x8DFB] = TODO # GL_MAX_VERTEX_UNIFORM_VECTORS
# _m[0x825B] = TODO # GL_MAX_VIEWPORTS
# _m[0x8E5B] = TODO # GL_MIN_FRAGMENT_INTERPOLATION_OFFSET
# _m[0x8E5B] = TODO # GL_MIN_FRAGMENT_INTERPOLATION_OFFSET_NV
# _m[0x90BC] = TODO # GL_MIN_MAP_BUFFER_ALIGNMENT
# _m[0x8904] = TODO # GL_MIN_PROGRAM_TEXEL_OFFSET_EXT
# _m[0x8E5E] = TODO # GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_ARB
# _m[0x8E5E] = TODO # GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_NV
# _m[0x8C37] = TODO # GL_MIN_SAMPLE_SHADING_VALUE_ARB
# _m[0x8E12] = TODO # GL_MULTISAMPLE_COVERAGE_MODES_NV
# _m[0x8261] = TODO # GL_NO_RESET_NOTIFICATION_ARB
# _m[0x87FE] = TODO # GL_NUM_PROGRAM_BINARY_FORMATS
# _m[0x8DF9] = TODO # GL_NUM_SHADER_BINARY_FORMATS
# _m[0x912D] = TODO # GL_PACK_COMPRESSED_BLOCK_DEPTH
# _m[0x912C] = TODO # GL_PACK_COMPRESSED_BLOCK_HEIGHT
# _m[0x912E] = TODO # GL_PACK_COMPRESSED_BLOCK_SIZE
# _m[0x912B] = TODO # GL_PACK_COMPRESSED_BLOCK_WIDTH
# _m[0x8A15] = TODO # GL_PACK_ROW_BYTES_APPLE
# _m[0x80EF] = TODO # GL_PARAMETER_BUFFER_BINDING_ARB
# _m[0x8E73] = TODO # GL_PATCH_DEFAULT_INNER_LEVEL
# _m[0x8E74] = TODO # GL_PATCH_DEFAULT_OUTER_LEVEL
# _m[0x8E72] = TODO # GL_PATCH_VERTICES
# _m[0x90BF] = TODO # GL_PATH_COVER_DEPTH_FUNC_NV
# _m[0x90AB] = TODO # GL_PATH_ERROR_POSITION_NV
# _m[0x90AC] = TODO # GL_PATH_FOG_GEN_MODE_NV
# _m[0x90BD] = TODO # GL_PATH_STENCIL_DEPTH_OFFSET_FACTOR_NV
# _m[0x90BE] = TODO # GL_PATH_STENCIL_DEPTH_OFFSET_UNITS_NV
# _m[0x90B7] = TODO # GL_PATH_STENCIL_FUNC_NV
# _m[0x90B8] = TODO # GL_PATH_STENCIL_REF_NV
# _m[0x90B9] = TODO # GL_PATH_STENCIL_VALUE_MASK_NV
# _m[0x87FF] = TODO # GL_PROGRAM_BINARY_FORMATS
# _m[0x825A] = TODO # GL_PROGRAM_PIPELINE_BINDING
# _m[0x9193] = TODO # GL_QUERY_BUFFER_BINDING
# _m[0x9193] = TODO # GL_QUERY_BUFFER_BINDING_AMD
# _m[0x8CAA] = TODO # GL_READ_FRAMEBUFFER_BINDING
# _m[0x8256] = TODO # GL_RESET_NOTIFICATION_STRATEGY_ARB
# _m[0x8820] = TODO # GL_RGBA_FLOAT_MODE_ATI
# _m[0x8D9E] = TODO # GL_RGBA_INTEGER_MODE_EXT
# _m[0x8C3C] = TODO # GL_RGBA_SIGNED_COMPONENTS_EXT
# _m[0x8919] = TODO # GL_SAMPLER_BINDING
# _m[0x8E51] = TODO # GL_SAMPLE_MASK
# _m[0x8E51] = TODO # GL_SAMPLE_MASK_NV
# _m[0x8C36] = TODO # GL_SAMPLE_SHADING_ARB
# _m[0x8DF8] = TODO # GL_SHADER_BINARY_FORMATS
# _m[0x8DFA] = TODO # GL_SHADER_COMPILER
# _m[0x90D3] = TODO # GL_SHADER_STORAGE_BUFFER_BINDING
# _m[0x90DF] = TODO # GL_SHADER_STORAGE_BUFFER_OFFSET_ALIGNMENT
# _m[0x91A9] = TODO # GL_SPARSE_TEXTURE_FULL_ARRAY_CUBE_MIPMAPS_ARB
# _m[0x8801] = TODO # GL_STENCIL_BACK_FAIL_ATI
# _m[0x8800] = TODO # GL_STENCIL_BACK_FUNC_ATI
# _m[0x874D] = TODO # GL_STENCIL_BACK_OP_VALUE_AMD
# _m[0x8802] = TODO # GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI
# _m[0x8803] = TODO # GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI
# _m[0x88F3] = TODO # GL_STENCIL_CLEAR_TAG_VALUE_EXT
# _m[0x874C] = TODO # GL_STENCIL_OP_VALUE_AMD
# _m[0x88F2] = TODO # GL_STENCIL_TAG_BITS_EXT
# _m[0x9004] = TODO # GL_TESSELLATION_MODE_AMD
# _m[0x891E] = TODO # GL_TESS_CONTROL_PROGRAM_NV
# _m[0x891F] = TODO # GL_TESS_EVALUATION_PROGRAM_NV
# _m[0x875D] = TODO # GL_TEXTURE_1D_STACK_BINDING_MESAX
# _m[0x8759] = TODO # GL_TEXTURE_1D_STACK_MESAX
# _m[0x875E] = TODO # GL_TEXTURE_2D_STACK_BINDING_MESAX
# _m[0x875A] = TODO # GL_TEXTURE_2D_STACK_MESAX
# _m[0x900A] = TODO # GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_ARB
# _m[0x919F] = TODO # GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT
# _m[0x8E28] = TODO # GL_TIMESTAMP
# _m[0x8E25] = TODO # GL_TRANSFORM_FEEDBACK_BINDING
# _m[0x8E24] = TODO # GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE
# _m[0x8E23] = TODO # GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED
# _m[0x9129] = TODO # GL_UNPACK_COMPRESSED_BLOCK_DEPTH
# _m[0x9128] = TODO # GL_UNPACK_COMPRESSED_BLOCK_HEIGHT
# _m[0x912A] = TODO # GL_UNPACK_COMPRESSED_BLOCK_SIZE
# _m[0x9127] = TODO # GL_UNPACK_COMPRESSED_BLOCK_WIDTH
# _m[0x8A16] = TODO # GL_UNPACK_ROW_BYTES_APPLE
# _m[0x87CF] = TODO # GL_VERTEX_SHADER_INSTRUCTIONS_EXT
# _m[0x87D1] = TODO # GL_VERTEX_SHADER_INVARIANTS_EXT
# _m[0x87D3] = TODO # GL_VERTEX_SHADER_LOCALS_EXT
# _m[0x87D2] = TODO # GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT
# _m[0x87D0] = TODO # GL_VERTEX_SHADER_VARIANTS_EXT
# _m[0x850C] = TODO # GL_VERTEX_WEIGHT_ARRAY_EXT
# _m[0x8510] = TODO # GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT
# _m[0x825D] = TODO # GL_VIEWPORT_BOUNDS_RANGE
# _m[0x825F] = TODO # GL_VIEWPORT_INDEX_PROVOKING_VERTEX
# _m[0x825C] = TODO # GL_VIEWPORT_SUBPIXEL_BITS
# _m[0x86AD] = TODO # GL_WEIGHT_ARRAY_ARB
_m[0x0D5B] = (1,) # GL_ACCUM_ALPHA_BITS
_m[0x0D5A] = (1,) # GL_ACCUM_BLUE_BITS
_m[0x0B80] = (4,) # GL_ACCUM_CLEAR_VALUE
_m[0x0D59] = (1,) # GL_ACCUM_GREEN_BITS
_m[0x0D58] = (1,) # GL_ACCUM_RED_BITS
_m[0x8911] = (1,) # GL_ACTIVE_STENCIL_FACE_EXT
_m[0x84E0] = (1,) # GL_ACTIVE_TEXTURE
_m[0x84E0] = (1,) # GL_ACTIVE_TEXTURE_ARB
_m[0x86A5] = (1,) # GL_ACTIVE_VERTEX_UNITS_ARB
_m[0x846E] = (2,) # GL_ALIASED_LINE_WIDTH_RANGE
_m[0x846D] = (2,) # GL_ALIASED_POINT_SIZE_RANGE
_m[0x0D1D] = (1,) # GL_ALPHA_BIAS
_m[0x0D55] = (1,) # GL_ALPHA_BITS
_m[0x8567] = (1,) # GL_ALPHA_MAX_CLAMP_INGR
_m[0x8563] = (1,) # GL_ALPHA_MIN_CLAMP_INGR
_m[0x0D1C] = (1,) # GL_ALPHA_SCALE
_m[0x0BC0] = (1,) # GL_ALPHA_TEST
_m[0x0BC1] = (1,) # GL_ALPHA_TEST_FUNC
_m[0x0BC2] = (1,) # GL_ALPHA_TEST_REF
_m[0x8894] = (1,) # GL_ARRAY_BUFFER_BINDING
_m[0x8894] = (1,) # GL_ARRAY_BUFFER_BINDING_ARB
_m[0x81A9] = (1,) # GL_ARRAY_ELEMENT_LOCK_COUNT_EXT
_m[0x81A8] = (1,) # GL_ARRAY_ELEMENT_LOCK_FIRST_EXT
_m[0x835D] = (1,) # GL_ASYNC_DRAW_PIXELS_SGIX
_m[0x832C] = (1,) # GL_ASYNC_HISTOGRAM_SGIX
_m[0x8329] = (1,) # GL_ASYNC_MARKER_SGIX
_m[0x835E] = (1,) # GL_ASYNC_READ_PIXELS_SGIX
_m[0x835C] = (1,) # GL_ASYNC_TEX_IMAGE_SGIX
_m[0x0BB0] = (1,) # GL_ATTRIB_STACK_DEPTH
_m[0x0D80] = (1,) # GL_AUTO_NORMAL
_m[0x0C00] = (1,) # GL_AUX_BUFFERS
_m[0x8441] = (1,) # GL_BINORMAL_ARRAY_STRIDE_EXT
_m[0x8440] = (1,) # GL_BINORMAL_ARRAY_TYPE_EXT
_m[0x0BE2] = (1,) # GL_BLEND
_m[0x8005] = (4,) # GL_BLEND_COLOR
_m[0x8005] = (4,) # GL_BLEND_COLOR_EXT
_m[0x0BE0] = (1,) # GL_BLEND_DST
_m[0x80CA] = (1,) # GL_BLEND_DST_ALPHA
_m[0x80CA] = (1,) # GL_BLEND_DST_ALPHA_EXT
_m[0x80C8] = (1,) # GL_BLEND_DST_RGB
_m[0x80C8] = (1,) # GL_BLEND_DST_RGB_EXT
_m[0x883D] = (1,) # GL_BLEND_EQUATION_ALPHA
_m[0x883D] = (1,) # GL_BLEND_EQUATION_ALPHA_EXT
_m[0x8009] = (1,) # GL_BLEND_EQUATION_EXT
_m[0x8009] = (1,) # GL_BLEND_EQUATION_RGB
_m[0x0BE1] = (1,) # GL_BLEND_SRC
_m[0x80CB] = (1,) # GL_BLEND_SRC_ALPHA
_m[0x80CB] = (1,) # GL_BLEND_SRC_ALPHA_EXT
_m[0x80C9] = (1,) # GL_BLEND_SRC_RGB
_m[0x80C9] = (1,) # GL_BLEND_SRC_RGB_EXT
_m[0x0D1B] = (1,) # GL_BLUE_BIAS
_m[0x0D54] = (1,) # GL_BLUE_BITS
_m[0x8566] = (1,) # GL_BLUE_MAX_CLAMP_INGR
_m[0x8562] = (1,) # GL_BLUE_MIN_CLAMP_INGR
_m[0x0D1A] = (1,) # GL_BLUE_SCALE
_m[0x891B] = (1,) # GL_CLAMP_FRAGMENT_COLOR_ARB
_m[0x891C] = (1,) # GL_CLAMP_READ_COLOR_ARB
_m[0x891A] = (1,) # GL_CLAMP_VERTEX_COLOR_ARB
_m[0x84E1] = (1,) # GL_CLIENT_ACTIVE_TEXTURE
_m[0x84E1] = (1,) # GL_CLIENT_ACTIVE_TEXTURE_ARB
_m[0x0BB1] = (1,) # GL_CLIENT_ATTRIB_STACK_DEPTH
_m[0x3000] = (1,) # GL_CLIP_PLANE0
_m[0x3001] = (1,) # GL_CLIP_PLANE1
_m[0x3002] = (1,) # GL_CLIP_PLANE2
_m[0x3003] = (1,) # GL_CLIP_PLANE3
_m[0x3004] = (1,) # GL_CLIP_PLANE4
_m[0x3005] = (1,) # GL_CLIP_PLANE5
_m[0x80F0] = (1,) # GL_CLIP_VOLUME_CLIPPING_HINT_EXT
_m[0x8975] = (1,) # GL_COLOR_ALPHA_PAIRING_ATI
_m[0x8076] = (1,) # GL_COLOR_ARRAY
_m[0x8898] = (1,) # GL_COLOR_ARRAY_BUFFER_BINDING
_m[0x8898] = (1,) # GL_COLOR_ARRAY_BUFFER_BINDING_ARB
_m[0x8084] = (1,) # GL_COLOR_ARRAY_COUNT_EXT
_m[0x8F2D] = (1,) # GL_COLOR_ARRAY_LENGTH_NV
_m[0x8081] = (1,) # GL_COLOR_ARRAY_SIZE
_m[0x8081] = (1,) # GL_COLOR_ARRAY_SIZE_EXT
_m[0x8083] = (1,) # GL_COLOR_ARRAY_STRIDE
_m[0x8083] = (1,) # GL_COLOR_ARRAY_STRIDE_EXT
_m[0x8082] = (1,) # GL_COLOR_ARRAY_TYPE
_m[0x8082] = (1,) # GL_COLOR_ARRAY_TYPE_EXT
_m[0x8835] = (4,) # GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI
_m[0x0C22] = (4,) # GL_COLOR_CLEAR_VALUE
_m[0x0BF2] = (1,) # GL_COLOR_LOGIC_OP
_m[0x0B57] = (1,) # GL_COLOR_MATERIAL
_m[0x0B55] = (1,) # GL_COLOR_MATERIAL_FACE
_m[0x0B56] = (1,) # GL_COLOR_MATERIAL_PARAMETER
_m[0x80B1] = (4, 4) # GL_COLOR_MATRIX
_m[0x80B1] = (4,4) # GL_COLOR_MATRIX_SGI
_m[0x80B2] = (1,) # GL_COLOR_MATRIX_STACK_DEPTH
_m[0x80B2] = (1,) # GL_COLOR_MATRIX_STACK_DEPTH_SGI
_m[0x8458] = (1,) # GL_COLOR_SUM
_m[0x8458] = (1,) # GL_COLOR_SUM_ARB
_m[0x854F] = (1,) # GL_COLOR_SUM_CLAMP_NV
_m[0x8458] = (1,) # GL_COLOR_SUM_EXT
_m[0x80D0] = (1,) # GL_COLOR_TABLE
_m[0x80D0] = (1,) # GL_COLOR_TABLE_SGI
_m[0x0C23] = (4,) # GL_COLOR_WRITEMASK
_m[0x86A3] = (1,) # GL_COMPRESSED_TEXTURE_FORMATS
_m[0x86A3] = (1,) # GL_COMPRESSED_TEXTURE_FORMATS_ARB
_m[0x8010] = (1,) # GL_CONVOLUTION_1D
_m[0x8010] = (1,) # GL_CONVOLUTION_1D_EXT
_m[0x8011] = (1,) # GL_CONVOLUTION_2D
_m[0x8011] = (1,) # GL_CONVOLUTION_2D_EXT
_m[0x0B44] = (1,) # GL_CULL_FACE
_m[0x0B45] = (1,) # GL_CULL_FACE_MODE
_m[0x843C] = (1,) # GL_CURRENT_BINORMAL_EXT
_m[0x0B00] = (4,) # GL_CURRENT_COLOR
_m[0x8453] = (1,) # GL_CURRENT_FOG_COORDINATE
_m[0x8453] = (1,) # GL_CURRENT_FOG_COORDINATE_EXT
_m[0x0B01] = (1,) # GL_CURRENT_INDEX
_m[0x8641] = (4, 4) # GL_CURRENT_MATRIX_ARB
_m[0x8641] = (4, 4) # GL_CURRENT_MATRIX_NV
_m[0x8640] = (1,) # GL_CURRENT_MATRIX_STACK_DEPTH_ARB
_m[0x8640] = (1,) # GL_CURRENT_MATRIX_STACK_DEPTH_NV
_m[0x0B02] = (3,) # GL_CURRENT_NORMAL
_m[0x8865] = (1,) # GL_CURRENT_OCCLUSION_QUERY_ID_NV
_m[0x8B8D] = (1,) # GL_CURRENT_PROGRAM
_m[0x0B04] = (4,) # GL_CURRENT_RASTER_COLOR
_m[0x0B09] = (1,) # GL_CURRENT_RASTER_DISTANCE
_m[0x0B05] = (1,) # GL_CURRENT_RASTER_INDEX
_m[0x0B07] = (4,) # GL_CURRENT_RASTER_POSITION
_m[0x0B08] = (1,) # GL_CURRENT_RASTER_POSITION_VALID
_m[0x0B06] = (4,) # GL_CURRENT_RASTER_TEXTURE_COORDS
_m[0x8459] = (4,) # GL_CURRENT_SECONDARY_COLOR
_m[0x8459] = (1,) # GL_CURRENT_SECONDARY_COLOR_EXT
_m[0x843B] = (1,) # GL_CURRENT_TANGENT_EXT
_m[0x0B03] = (4,) # GL_CURRENT_TEXTURE_COORDS
_m[0x850B] = (1,) # GL_CURRENT_VERTEX_WEIGHT_EXT
_m[0x86A8] = (1,) # GL_CURRENT_WEIGHT_ARB
_m[0x0D1F] = (1,) # GL_DEPTH_BIAS
_m[0x0D56] = (1,) # GL_DEPTH_BITS
_m[0x8891] = (1,) # GL_DEPTH_BOUNDS_EXT
_m[0x8890] = (1,) # GL_DEPTH_BOUNDS_TEST_EXT
_m[0x864F] = (1,) # GL_DEPTH_CLAMP_NV
_m[0x0B73] = (1,) # GL_DEPTH_CLEAR_VALUE
_m[0x0B74] = (1,) # GL_DEPTH_FUNC
_m[0x0B70] = (2,) # GL_DEPTH_RANGE
_m[0x0D1E] = (1,) # GL_DEPTH_SCALE
_m[0x0B71] = (1,) # GL_DEPTH_TEST
_m[0x0B72] = (1,) # GL_DEPTH_WRITEMASK
_m[0x0BD0] = (1,) # GL_DITHER
_m[0x0C32] = (1,) # GL_DOUBLEBUFFER
_m[0x0C01] = (1,) # GL_DRAW_BUFFER
_m[0x8825] = (1,) # GL_DRAW_BUFFER0
_m[0x8825] = (1,) # GL_DRAW_BUFFER0_ARB
_m[0x8825] = (1,) # GL_DRAW_BUFFER0_ATI
_m[0x8826] = (1,) # GL_DRAW_BUFFER1
_m[0x882F] = (1,) # GL_DRAW_BUFFER10
_m[0x882F] = (1,) # GL_DRAW_BUFFER10_ARB
_m[0x882F] = (1,) # GL_DRAW_BUFFER10_ATI
_m[0x8830] = (1,) # GL_DRAW_BUFFER11
_m[0x8830] = (1,) # GL_DRAW_BUFFER11_ARB
_m[0x8830] = (1,) # GL_DRAW_BUFFER11_ATI
_m[0x8831] = (1,) # GL_DRAW_BUFFER12
_m[0x8831] = (1,) # GL_DRAW_BUFFER12_ARB
_m[0x8831] = (1,) # GL_DRAW_BUFFER12_ATI
_m[0x8832] = (1,) # GL_DRAW_BUFFER13
_m[0x8832] = (1,) # GL_DRAW_BUFFER13_ARB
_m[0x8832] = (1,) # GL_DRAW_BUFFER13_ATI
_m[0x8833] = (1,) # GL_DRAW_BUFFER14
_m[0x8833] = (1,) # GL_DRAW_BUFFER14_ARB
_m[0x8833] = (1,) # GL_DRAW_BUFFER14_ATI
_m[0x8834] = (1,) # GL_DRAW_BUFFER15
_m[0x8834] = (1,) # GL_DRAW_BUFFER15_ARB
_m[0x8834] = (1,) # GL_DRAW_BUFFER15_ATI
_m[0x8826] = (1,) # GL_DRAW_BUFFER1_ARB
_m[0x8826] = (1,) # GL_DRAW_BUFFER1_ATI
_m[0x8827] = (1,) # GL_DRAW_BUFFER2
_m[0x8827] = (1,) # GL_DRAW_BUFFER2_ARB
_m[0x8827] = (1,) # GL_DRAW_BUFFER2_ATI
_m[0x8828] = (1,) # GL_DRAW_BUFFER3
_m[0x8828] = (1,) # GL_DRAW_BUFFER3_ARB
_m[0x8828] = (1,) # GL_DRAW_BUFFER3_ATI
_m[0x8829] = (1,) # GL_DRAW_BUFFER4
_m[0x8829] = (1,) # GL_DRAW_BUFFER4_ARB
_m[0x8829] = (1,) # GL_DRAW_BUFFER4_ATI
_m[0x882A] = (1,) # GL_DRAW_BUFFER5
_m[0x882A] = (1,) # GL_DRAW_BUFFER5_ARB
_m[0x882A] = (1,) # GL_DRAW_BUFFER5_ATI
_m[0x882B] = (1,) # GL_DRAW_BUFFER6
_m[0x882B] = (1,) # GL_DRAW_BUFFER6_ARB
_m[0x882B] = (1,) # GL_DRAW_BUFFER6_ATI
_m[0x882C] = (1,) # GL_DRAW_BUFFER7
_m[0x882C] = (1,) # GL_DRAW_BUFFER7_ARB
_m[0x882C] = (1,) # GL_DRAW_BUFFER7_ATI
_m[0x882D] = (1,) # GL_DRAW_BUFFER8
_m[0x882D] = (1,) # GL_DRAW_BUFFER8_ARB
_m[0x882D] = (1,) # GL_DRAW_BUFFER8_ATI
_m[0x882E] = (1,) # GL_DRAW_BUFFER9
_m[0x882E] = (1,) # GL_DRAW_BUFFER9_ARB
_m[0x882E] = (1,) # GL_DRAW_BUFFER9_ATI
_m[0x8716] = (1,) # GL_DS_BIAS_NV
_m[0x8710] = (1,) # GL_DS_SCALE_NV
_m[0x8717] = (1,) # GL_DT_BIAS_NV
_m[0x8711] = (1,) # GL_DT_SCALE_NV
_m[0x0B43] = (1,) # GL_EDGE_FLAG
_m[0x8079] = (1,) # GL_EDGE_FLAG_ARRAY
_m[0x889B] = (1,) # GL_EDGE_FLAG_ARRAY_BUFFER_BINDING
_m[0x889B] = (1,) # GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB
_m[0x808D] = (1,) # GL_EDGE_FLAG_ARRAY_COUNT_EXT
_m[0x8079] = (1,) # GL_EDGE_FLAG_ARRAY_EXT
_m[0x8F30] = (1,) # GL_EDGE_FLAG_ARRAY_LENGTH_NV
_m[0x808C] = (1,) # GL_EDGE_FLAG_ARRAY_STRIDE
_m[0x808C] = (1,) # GL_EDGE_FLAG_ARRAY_STRIDE_EXT
_m[0x8895] = (1,) # GL_ELEMENT_ARRAY_BUFFER_BINDING
_m[0x8895] = (1,) # GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB
_m[0x8F33] = (1,) # GL_ELEMENT_ARRAY_LENGTH_NV
_m[0x8A0D] = (1,) # GL_ELEMENT_ARRAY_TYPE_APPLE
_m[0x8769] = (1,) # GL_ELEMENT_ARRAY_TYPE_ATI
_m[0x86C5] = (1,) # GL_EVAL_FRACTIONAL_TESSELLATION_NV
_m[0x86C6] = (1,) # GL_EVAL_VERTEX_ATTRIB0_NV
_m[0x86D0] = (1,) # GL_EVAL_VERTEX_ATTRIB10_NV
_m[0x86D1] = (1,) # GL_EVAL_VERTEX_ATTRIB11_NV
_m[0x86D2] = (1,) # GL_EVAL_VERTEX_ATTRIB12_NV
_m[0x86D3] = (1,) # GL_EVAL_VERTEX_ATTRIB13_NV
_m[0x86D4] = (1,) # GL_EVAL_VERTEX_ATTRIB14_NV
_m[0x86D5] = (1,) # GL_EVAL_VERTEX_ATTRIB15_NV
_m[0x86C7] = (1,) # GL_EVAL_VERTEX_ATTRIB1_NV
_m[0x86C8] = (1,) # GL_EVAL_VERTEX_ATTRIB2_NV
_m[0x86C9] = (1,) # GL_EVAL_VERTEX_ATTRIB3_NV
_m[0x86CA] = (1,) # GL_EVAL_VERTEX_ATTRIB4_NV
_m[0x86CB] = (1,) # GL_EVAL_VERTEX_ATTRIB5_NV
_m[0x86CC] = (1,) # GL_EVAL_VERTEX_ATTRIB6_NV
_m[0x86CD] = (1,) # GL_EVAL_VERTEX_ATTRIB7_NV
_m[0x86CE] = (1,) # GL_EVAL_VERTEX_ATTRIB8_NV
_m[0x86CF] = (1,) # GL_EVAL_VERTEX_ATTRIB9_NV
_m[0x0DF1] = (1,) # GL_FEEDBACK_BUFFER_SIZE
_m[0x0DF2] = (1,) # GL_FEEDBACK_BUFFER_TYPE
_m[0x888D] = (4,) # GL_FLOAT_CLEAR_COLOR_VALUE_NV
_m[0x888E] = (1,) # GL_FLOAT_RGBA_MODE_NV
_m[0x0B60] = (1,) # GL_FOG
_m[0x0B66] = (4,) # GL_FOG_COLOR
_m[0x889D] = (1,) # GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB
_m[0x8455] = (1,) # GL_FOG_COORDINATE_ARRAY_STRIDE_EXT
_m[0x8454] = (1,) # GL_FOG_COORDINATE_ARRAY_TYPE_EXT
_m[0x8457] = (1,) # GL_FOG_COORD_ARRAY
_m[0x889D] = (1,) # GL_FOG_COORD_ARRAY_BUFFER_BINDING
_m[0x8F32] = (1,) # GL_FOG_COORD_ARRAY_LENGTH_NV
_m[0x8455] = (1,) # GL_FOG_COORD_ARRAY_STRIDE
_m[0x8454] = (1,) # GL_FOG_COORD_ARRAY_TYPE
_m[0x8450] = (1,) # GL_FOG_COORD_SRC
_m[0x0B62] = (1,) # GL_FOG_DENSITY
_m[0x855A] = (1,) # GL_FOG_DISTANCE_MODE_NV
_m[0x0B64] = (1,) # GL_FOG_END
_m[0x0C54] = (1,) # GL_FOG_HINT
_m[0x0B61] = (1,) # GL_FOG_INDEX
_m[0x0B65] = (1,) # GL_FOG_MODE
_m[0x0B63] = (1,) # GL_FOG_START
_m[0x8804] = (1,) # GL_FRAGMENT_PROGRAM_ARB
_m[0x8873] = (1,) # GL_FRAGMENT_PROGRAM_BINDING_NV
_m[0x8870] = (1,) # GL_FRAGMENT_PROGRAM_NV
_m[0x8920] = (1,) # GL_FRAGMENT_SHADER_ATI
_m[0x8B8B] = (1,) # GL_FRAGMENT_SHADER_DERIVATIVE_HINT
_m[0x8B8B] = (1,) # GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB
_m[0x8CA6] = (1,) # GL_FRAMEBUFFER_BINDING_EXT
_m[0x0B46] = (1,) # GL_FRONT_FACE
_m[0x8192] = (1,) # GL_GENERATE_MIPMAP_HINT
_m[0x8192] = (1,) # GL_GENERATE_MIPMAP_HINT_SGIS
_m[0x81DA] = (1,) # GL_GLOBAL_ALPHA_FACTOR_SUN
_m[0x0D19] = (1,) # GL_GREEN_BIAS
_m[0x0D53] = (1,) # GL_GREEN_BITS
_m[0x8565] = (1,) # GL_GREEN_MAX_CLAMP_INGR
_m[0x8561] = (1,) # GL_GREEN_MIN_CLAMP_INGR
_m[0x0D18] = (1,) # GL_GREEN_SCALE
_m[0x8024] = (1,) # GL_HISTOGRAM
_m[0x8024] = (1,) # GL_HISTOGRAM_EXT
_m[0x8714] = (1,) # GL_HI_BIAS_NV
_m[0x870E] = (1,) # GL_HI_SCALE_NV
_m[0x8077] = (1,) # GL_INDEX_ARRAY
_m[0x8899] = (1,) # GL_INDEX_ARRAY_BUFFER_BINDING
_m[0x8899] = (1,) # GL_INDEX_ARRAY_BUFFER_BINDING_ARB
_m[0x8087] = (1,) # GL_INDEX_ARRAY_COUNT_EXT
_m[0x8077] = (1,) # GL_INDEX_ARRAY_EXT
_m[0x8F2E] = (1,) # GL_INDEX_ARRAY_LENGTH_NV
_m[0x8086] = (1,) # GL_INDEX_ARRAY_STRIDE
_m[0x8086] = (1,) # GL_INDEX_ARRAY_STRIDE_EXT
_m[0x8085] = (1,) # GL_INDEX_ARRAY_TYPE
_m[0x8085] = (1,) # GL_INDEX_ARRAY_TYPE_EXT
_m[0x0D51] = (1,) # GL_INDEX_BITS
_m[0x0C20] = (1,) # GL_INDEX_CLEAR_VALUE
_m[0x0BF1] = (1,) # GL_INDEX_LOGIC_OP
_m[0x0C30] = (1,) # GL_INDEX_MODE
_m[0x0D13] = (1,) # GL_INDEX_OFFSET
_m[0x0D12] = (1,) # GL_INDEX_SHIFT
_m[0x0C21] = (1,) # GL_INDEX_WRITEMASK
_m[0x8980] = (1,) # GL_INTERLACE_OML
_m[0x8568] = (1,) # GL_INTERLACE_READ_INGR
_m[0x8981] = (1,) # GL_INTERLACE_READ_OML
_m[0x8094] = (1,) # GL_INTERLACE_SGIX
_m[0x4000] = (1,) # GL_LIGHT0
_m[0x4001] = (1,) # GL_LIGHT1
_m[0x4002] = (1,) # GL_LIGHT2
_m[0x4003] = (1,) # GL_LIGHT3
_m[0x4004] = (1,) # GL_LIGHT4
_m[0x4005] = (1,) # GL_LIGHT5
_m[0x4006] = (1,) # GL_LIGHT6
_m[0x4007] = (1,) # GL_LIGHT7
_m[0x0B50] = (1,) # GL_LIGHTING
_m[0x0B53] = (4,) # GL_LIGHT_MODEL_AMBIENT
_m[0x81F8] = (1,) # GL_LIGHT_MODEL_COLOR_CONTROL
_m[0x81F8] = (1,) # GL_LIGHT_MODEL_COLOR_CONTROL_EXT
_m[0x0B51] = (1,) # GL_LIGHT_MODEL_LOCAL_VIEWER
_m[0x0B52] = (1,) # GL_LIGHT_MODEL_TWO_SIDE
_m[0x0B20] = (1,) # GL_LINE_SMOOTH
_m[0x0C52] = (1,) # GL_LINE_SMOOTH_HINT
_m[0x0B24] = (1,) # GL_LINE_STIPPLE
_m[0x0B25] = (1,) # GL_LINE_STIPPLE_PATTERN
_m[0x0B26] = (1,) # GL_LINE_STIPPLE_REPEAT
_m[0x0B21] = (1,) # GL_LINE_WIDTH
_m[0x0B23] = (1,) # GL_LINE_WIDTH_GRANULARITY
_m[0x0B22] = (2,) # GL_LINE_WIDTH_RANGE
_m[0x0B32] = (1,) # GL_LIST_BASE
_m[0x0B33] = (1,) # GL_LIST_INDEX
_m[0x0B30] = (1,) # GL_LIST_MODE
_m[0x0BF0] = (1,) # GL_LOGIC_OP_MODE
_m[0x8715] = (1,) # GL_LO_BIAS_NV
_m[0x870F] = (1,) # GL_LO_SCALE_NV
_m[0x8718] = (1,) # GL_MAGNITUDE_BIAS_NV
_m[0x8712] = (1,) # GL_MAGNITUDE_SCALE_NV
_m[0x0D90] = (1,) # GL_MAP1_COLOR_4
_m[0x0DD0] = (2,) # GL_MAP1_GRID_DOMAIN
_m[0x0DD1] = (1,) # GL_MAP1_GRID_SEGMENTS
_m[0x0D91] = (1,) # GL_MAP1_INDEX
_m[0x0D92] = (1,) # GL_MAP1_NORMAL
_m[0x0D93] = (1,) # GL_MAP1_TEXTURE_COORD_1
_m[0x0D94] = (1,) # GL_MAP1_TEXTURE_COORD_2
_m[0x0D95] = (1,) # GL_MAP1_TEXTURE_COORD_3
_m[0x0D96] = (1,) # GL_MAP1_TEXTURE_COORD_4
_m[0x0D97] = (1,) # GL_MAP1_VERTEX_3
_m[0x0D98] = (1,) # GL_MAP1_VERTEX_4
_m[0x8660] = (4,) # GL_MAP1_VERTEX_ATTRIB0_4_NV
_m[0x866A] = (4,) # GL_MAP1_VERTEX_ATTRIB10_4_NV
_m[0x866B] = (4,) # GL_MAP1_VERTEX_ATTRIB11_4_NV
_m[0x866C] = (4,) # GL_MAP1_VERTEX_ATTRIB12_4_NV
_m[0x866D] = (4,) # GL_MAP1_VERTEX_ATTRIB13_4_NV
_m[0x866E] = (4,) # GL_MAP1_VERTEX_ATTRIB14_4_NV
_m[0x866F] = (4,) # GL_MAP1_VERTEX_ATTRIB15_4_NV
_m[0x8661] = (4,) # GL_MAP1_VERTEX_ATTRIB1_4_NV
_m[0x8662] = (4,) # GL_MAP1_VERTEX_ATTRIB2_4_NV
_m[0x8663] = (4,) # GL_MAP1_VERTEX_ATTRIB3_4_NV
_m[0x8664] = (4,) # GL_MAP1_VERTEX_ATTRIB4_4_NV
_m[0x8665] = (4,) # GL_MAP1_VERTEX_ATTRIB5_4_NV
_m[0x8666] = (4,) # GL_MAP1_VERTEX_ATTRIB6_4_NV
_m[0x8667] = (4,) # GL_MAP1_VERTEX_ATTRIB7_4_NV
_m[0x8668] = (4,) # GL_MAP1_VERTEX_ATTRIB8_4_NV
_m[0x8669] = (4,) # GL_MAP1_VERTEX_ATTRIB9_4_NV
_m[0x0DB0] = (1,) # GL_MAP2_COLOR_4
_m[0x0DD2] = (4,) # GL_MAP2_GRID_DOMAIN
_m[0x0DD3] = (2,) # GL_MAP2_GRID_SEGMENTS
_m[0x0DB1] = (1,) # GL_MAP2_INDEX
_m[0x0DB2] = (1,) # GL_MAP2_NORMAL
_m[0x0DB3] = (1,) # GL_MAP2_TEXTURE_COORD_1
_m[0x0DB4] = (1,) # GL_MAP2_TEXTURE_COORD_2
_m[0x0DB5] = (1,) # GL_MAP2_TEXTURE_COORD_3
_m[0x0DB6] = (1,) # GL_MAP2_TEXTURE_COORD_4
_m[0x0DB7] = (1,) # GL_MAP2_VERTEX_3
_m[0x0DB8] = (1,) # GL_MAP2_VERTEX_4
_m[0x8670] = (4,) # GL_MAP2_VERTEX_ATTRIB0_4_NV
_m[0x867A] = (4,) # GL_MAP2_VERTEX_ATTRIB10_4_NV
_m[0x867B] = (4,) # GL_MAP2_VERTEX_ATTRIB11_4_NV
_m[0x867C] = (4,) # GL_MAP2_VERTEX_ATTRIB12_4_NV
_m[0x867D] = (4,) # GL_MAP2_VERTEX_ATTRIB13_4_NV
_m[0x867E] = (4,) # GL_MAP2_VERTEX_ATTRIB14_4_NV
_m[0x867F] = (4,) # GL_MAP2_VERTEX_ATTRIB15_4_NV
_m[0x8671] = (4,) # GL_MAP2_VERTEX_ATTRIB1_4_NV
_m[0x8672] = (4,) # GL_MAP2_VERTEX_ATTRIB2_4_NV
_m[0x8673] = (4,) # GL_MAP2_VERTEX_ATTRIB3_4_NV
_m[0x8674] = (4,) # GL_MAP2_VERTEX_ATTRIB4_4_NV
_m[0x8675] = (4,) # GL_MAP2_VERTEX_ATTRIB5_4_NV
_m[0x8676] = (4,) # GL_MAP2_VERTEX_ATTRIB6_4_NV
_m[0x8677] = (4,) # GL_MAP2_VERTEX_ATTRIB7_4_NV
_m[0x8678] = (4,) # GL_MAP2_VERTEX_ATTRIB8_4_NV
_m[0x8679] = (4,) # GL_MAP2_VERTEX_ATTRIB9_4_NV
_m[0x0D10] = (1,) # GL_MAP_COLOR
_m[0x0D11] = (1,) # GL_MAP_STENCIL
_m[0x8846] = (1,) # GL_MATRIX_INDEX_ARRAY_SIZE_ARB
_m[0x8848] = (1,) # GL_MATRIX_INDEX_ARRAY_STRIDE_ARB
_m[0x8847] = (1,) # GL_MATRIX_INDEX_ARRAY_TYPE_ARB
_m[0x0BA0] = (1,) # GL_MATRIX_MODE
_m[0x8073] = (1,) # GL_MAX_3D_TEXTURE_SIZE
_m[0x8073] = (1,) # GL_MAX_3D_TEXTURE_SIZE_EXT
_m[0x88FF] = (1,) # GL_MAX_ARRAY_TEXTURE_LAYERS_EXT
_m[0x8360] = (1,) # GL_MAX_ASYNC_DRAW_PIXELS_SGIX
_m[0x832D] = (1,) # GL_MAX_ASYNC_HISTOGRAM_SGIX
_m[0x8361] = (1,) # GL_MAX_ASYNC_READ_PIXELS_SGIX
_m[0x835F] = (1,) # GL_MAX_ASYNC_TEX_IMAGE_SGIX
_m[0x0D35] = (1,) # GL_MAX_ATTRIB_STACK_DEPTH
_m[0x8DED] = (1,) # GL_MAX_BINDABLE_UNIFORM_SIZE_EXT
_m[0x0D3B] = (1,) # GL_MAX_CLIENT_ATTRIB_STACK_DEPTH
_m[0x0D32] = (1,) # GL_MAX_CLIP_PLANES
_m[0x8CDF] = (1,) # GL_MAX_COLOR_ATTACHMENTS
_m[0x8CDF] = (1,) # GL_MAX_COLOR_ATTACHMENTS_EXT
_m[0x80B3] = (1,) # GL_MAX_COLOR_MATRIX_STACK_DEPTH
_m[0x80B3] = (1,) # GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI
_m[0x910E] = (1,) # GL_MAX_COLOR_TEXTURE_SAMPLES
_m[0x8A33] = (1,) # GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS
_m[0x8A32] = (1,) # GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS
_m[0x8B4D] = (1,) # GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
_m[0x8B4D] = (1,) # GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB
_m[0x8A2E] = (1,) # GL_MAX_COMBINED_UNIFORM_BLOCKS
_m[0x8A31] = (1,) # GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS
_m[0x851C] = (1,) # GL_MAX_CUBE_MAP_TEXTURE_SIZE
_m[0x851C] = (1,) # GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB
_m[0x910F] = (1,) # GL_MAX_DEPTH_TEXTURE_SAMPLES
_m[0x8824] = (1,) # GL_MAX_DRAW_BUFFERS
_m[0x8824] = (1,) # GL_MAX_DRAW_BUFFERS_ARB
_m[0x8824] = (1,) # GL_MAX_DRAW_BUFFERS_ATI
_m[0x80E9] = (1,) # GL_MAX_ELEMENTS_INDICES
_m[0x80E8] = (1,) # GL_MAX_ELEMENTS_VERTICES
_m[0x0D30] = (1,) # GL_MAX_EVAL_ORDER
_m[0x8DE3] = (1,) # GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT
_m[0x8868] = (1,) # GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV
_m[0x8A2D] = (1,) # GL_MAX_FRAGMENT_UNIFORM_BLOCKS
_m[0x8B49] = (1,) # GL_MAX_FRAGMENT_UNIFORM_COMPONENTS
_m[0x8B49] = (1,) # GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB
_m[0x854D] = (1,) # GL_MAX_GENERAL_COMBINERS_NV
_m[0x8DE4] = (1,) # GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT
_m[0x8DE0] = (1,) # GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB
_m[0x8DE0] = (1,) # GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT
_m[0x8C29] = (1,) # GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB
_m[0x8C29] = (1,) # GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT
_m[0x8DE1] = (1,) # GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB
_m[0x8DE1] = (1,) # GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT
_m[0x8A2C] = (1,) # GL_MAX_GEOMETRY_UNIFORM_BLOCKS
_m[0x8DDF] = (1,) # GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB
_m[0x8DDF] = (1,) # GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT
_m[0x8DDD] = (1,) # GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB
_m[0x8DDD] = (1,) # GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT
_m[0x9110] = (1,) # GL_MAX_INTEGER_SAMPLES
_m[0x0D31] = (1,) # GL_MAX_LIGHTS
_m[0x0B31] = (1,) # GL_MAX_LIST_NESTING
_m[0x86D6] = (1,) # GL_MAX_MAP_TESSELLATION_NV
_m[0x8841] = (1,) # GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB
_m[0x0D36] = (1,) # GL_MAX_MODELVIEW_STACK_DEPTH
_m[0x8E11] = (1,) # GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV
_m[0x0D37] = (1,) # GL_MAX_NAME_STACK_DEPTH
_m[0x87CA] = (1,) # GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT
_m[0x87CE] = (1,) # GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT
_m[0x87CC] = (1,) # GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT
_m[0x87CB] = (1,) # GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT
_m[0x8842] = (1,) # GL_MAX_PALETTE_MATRICES_ARB
_m[0x0D34] = (1,) # GL_MAX_PIXEL_MAP_TABLE
_m[0x8337] = (1,) # GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT
_m[0x87F1] = (1,) # GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI
_m[0x862F] = (1,) # GL_MAX_PROGRAM_MATRICES_ARB
_m[0x862E] = (1,) # GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB
_m[0x0D38] = (1,) # GL_MAX_PROJECTION_STACK_DEPTH
_m[0x86D7] = (1,) # GL_MAX_RATIONAL_EVAL_ORDER_NV
_m[0x84F8] = (1,) # GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB
_m[0x84F8] = (1,) # GL_MAX_RECTANGLE_TEXTURE_SIZE_NV
_m[0x84E8] = (1,) # GL_MAX_RENDERBUFFER_SIZE
_m[0x84E8] = (1,) # GL_MAX_RENDERBUFFER_SIZE_EXT
_m[0x8D57] = (1,) # GL_MAX_SAMPLES
_m[0x8D57] = (1,) # GL_MAX_SAMPLES_EXT
_m[0x8E59] = (1,) # GL_MAX_SAMPLE_MASK_WORDS
_m[0x8E59] = (1,) # GL_MAX_SAMPLE_MASK_WORDS_NV
_m[0x8504] = (1,) # GL_MAX_SHININESS_NV
_m[0x8505] = (1,) # GL_MAX_SPOT_EXPONENT_NV
_m[0x8C2B] = (1,) # GL_MAX_TEXTURE_BUFFER_SIZE_ARB
_m[0x8C2B] = (1,) # GL_MAX_TEXTURE_BUFFER_SIZE_EXT
_m[0x8871] = (1,) # GL_MAX_TEXTURE_COORDS
_m[0x8871] = (1,) # GL_MAX_TEXTURE_COORDS_ARB
_m[0x8871] = (1,) # GL_MAX_TEXTURE_COORDS_NV
_m[0x8872] = (1,) # GL_MAX_TEXTURE_IMAGE_UNITS
_m[0x8872] = (1,) # GL_MAX_TEXTURE_IMAGE_UNITS_ARB
_m[0x8872] = (1,) # GL_MAX_TEXTURE_IMAGE_UNITS_NV
_m[0x84FD] = (1,) # GL_MAX_TEXTURE_LOD_BIAS
_m[0x84FD] = (1,) # GL_MAX_TEXTURE_LOD_BIAS_EXT
_m[0x84FF] = (1,) # GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT
_m[0x0D33] = (1,) # GL_MAX_TEXTURE_SIZE
_m[0x0D39] = (1,) # GL_MAX_TEXTURE_STACK_DEPTH
_m[0x84E2] = (1,) # GL_MAX_TEXTURE_UNITS
_m[0x84E2] = (1,) # GL_MAX_TEXTURE_UNITS_ARB
_m[0x862F] = (1,) # GL_MAX_TRACK_MATRICES_NV
_m[0x862E] = (1,) # GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV
_m[0x8A30] = (1,) # GL_MAX_UNIFORM_BLOCK_SIZE
_m[0x8A2F] = (1,) # GL_MAX_UNIFORM_BUFFER_BINDINGS
_m[0x8B4B] = (1,) # GL_MAX_VARYING_COMPONENTS_EXT
_m[0x8B4B] = (1,) # GL_MAX_VARYING_FLOATS
_m[0x8B4B] = (1,) # GL_MAX_VARYING_FLOATS_ARB
_m[0x8520] = (1,) # GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV
_m[0x8869] = (1,) # GL_MAX_VERTEX_ATTRIBS
_m[0x8869] = (1,) # GL_MAX_VERTEX_ATTRIBS_ARB
_m[0x8DE2] = (1,) # GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT
_m[0x87C5] = (1,) # GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT
_m[0x87C7] = (1,) # GL_MAX_VERTEX_SHADER_INVARIANTS_EXT
_m[0x87C9] = (1,) # GL_MAX_VERTEX_SHADER_LOCALS_EXT
_m[0x87C8] = (1,) # GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT
_m[0x87C6] = (1,) # GL_MAX_VERTEX_SHADER_VARIANTS_EXT
_m[0x876B] = (1,) # GL_MAX_VERTEX_STREAMS_ATI
_m[0x8B4C] = (1,) # GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS
_m[0x8B4C] = (1,) # GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB
_m[0x8A2B] = (1,) # GL_MAX_VERTEX_UNIFORM_BLOCKS
_m[0x8B4A] = (1,) # GL_MAX_VERTEX_UNIFORM_COMPONENTS
_m[0x8B4A] = (1,) # GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB
_m[0x86A4] = (1,) # GL_MAX_VERTEX_UNITS_ARB
_m[0x8DDE] = (1,) # GL_MAX_VERTEX_VARYING_COMPONENTS_ARB
_m[0x8DDE] = (1,) # GL_MAX_VERTEX_VARYING_COMPONENTS_EXT
_m[0x0D3A] = (2,) # GL_MAX_VIEWPORT_DIMS
_m[0x802E] = (1,) # GL_MINMAX
_m[0x802E] = (1,) # GL_MINMAX_EXT
_m[0x8506] = (4,4) # GL_MODELVIEW1_MATRIX_EXT
_m[0x8502] = (1,) # GL_MODELVIEW1_STACK_DEPTH_EXT
_m[0x0BA6] = (4, 4) # GL_MODELVIEW_MATRIX
_m[0x0BA3] = (1,) # GL_MODELVIEW_STACK_DEPTH
_m[0x809D] = (1,) # GL_MULTISAMPLE
_m[0x86B2] = (1,) # GL_MULTISAMPLE_3DFX
_m[0x809D] = (1,) # GL_MULTISAMPLE_ARB
_m[0x809D] = (1,) # GL_MULTISAMPLE_EXT
_m[0x8534] = (1,) # GL_MULTISAMPLE_FILTER_HINT_NV
_m[0x809D] = (1,) # GL_MULTISAMPLE_SGIS
_m[0x0D70] = (1,) # GL_NAME_STACK_DEPTH
_m[0x0BA1] = (1,) # GL_NORMALIZE
_m[0x8075] = (1,) # GL_NORMAL_ARRAY
_m[0x8897] = (1,) # GL_NORMAL_ARRAY_BUFFER_BINDING
_m[0x8897] = (1,) # GL_NORMAL_ARRAY_BUFFER_BINDING_ARB
_m[0x8080] = (1,) # GL_NORMAL_ARRAY_COUNT_EXT
_m[0x8075] = (1,) # GL_NORMAL_ARRAY_EXT
_m[0x8F2C] = (1,) # GL_NORMAL_ARRAY_LENGTH_NV
_m[0x807F] = (1,) # GL_NORMAL_ARRAY_STRIDE
_m[0x807F] = (1,) # GL_NORMAL_ARRAY_STRIDE_EXT
_m[0x807E] = (1,) # GL_NORMAL_ARRAY_TYPE
_m[0x807E] = (1,) # GL_NORMAL_ARRAY_TYPE_EXT
_m[0x86A2] = (1,) # GL_NUM_COMPRESSED_TEXTURE_FORMATS
_m[0x86A2] = (1,) # GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB
_m[0x896F] = (1,) # GL_NUM_FRAGMENT_CONSTANTS_ATI
_m[0x896E] = (1,) # GL_NUM_FRAGMENT_REGISTERS_ATI
_m[0x854E] = (1,) # GL_NUM_GENERAL_COMBINERS_NV
_m[0x8973] = (1,) # GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI
_m[0x8971] = (1,) # GL_NUM_INSTRUCTIONS_PER_PASS_ATI
_m[0x8972] = (1,) # GL_NUM_INSTRUCTIONS_TOTAL_ATI
_m[0x8974] = (1,) # GL_NUM_LOOPBACK_COMPONENTS_ATI
_m[0x8970] = (1,) # GL_NUM_PASSES_ATI
_m[0x8165] = (1,) # GL_OCCLUSION_TEST_HP
_m[0x8166] = (1,) # GL_OCCLUSION_TEST_RESULT_HP
_m[0x0D05] = (1,) # GL_PACK_ALIGNMENT
_m[0x800E] = (1,) # GL_PACK_CMYK_HINT_EXT
_m[0x806C] = (1,) # GL_PACK_IMAGE_HEIGHT
_m[0x806C] = (1,) # GL_PACK_IMAGE_HEIGHT_EXT
_m[0x8758] = (1,) # GL_PACK_INVERT_MESA
_m[0x0D01] = (1,) # GL_PACK_LSB_FIRST
_m[0x8984] = (1,) # GL_PACK_RESAMPLE_OML
_m[0x842C] = (1,) # GL_PACK_RESAMPLE_SGIX
_m[0x0D02] = (1,) # GL_PACK_ROW_LENGTH
_m[0x806B] = (1,) # GL_PACK_SKIP_IMAGES
_m[0x806B] = (1,) # GL_PACK_SKIP_IMAGES_EXT
_m[0x0D04] = (1,) # GL_PACK_SKIP_PIXELS
_m[0x0D03] = (1,) # GL_PACK_SKIP_ROWS
_m[0x0D00] = (1,) # GL_PACK_SWAP_BYTES
_m[0x83F4] = (1,) # GL_PARALLEL_ARRAYS_INTEL
_m[0x0C50] = (1,) # GL_PERSPECTIVE_CORRECTION_HINT
_m[0x8535] = (1,) # GL_PER_STAGE_CONSTANTS_NV
_m[0x8864] = (1,) # GL_PIXEL_COUNTER_BITS_NV
_m[0x0CB9] = (1,) # GL_PIXEL_MAP_A_TO_A_SIZE
_m[0x0CB8] = (1,) # GL_PIXEL_MAP_B_TO_B_SIZE
_m[0x0CB7] = (1,) # GL_PIXEL_MAP_G_TO_G_SIZE
_m[0x0CB5] = (1,) # GL_PIXEL_MAP_I_TO_A_SIZE
_m[0x0CB4] = (1,) # GL_PIXEL_MAP_I_TO_B_SIZE
_m[0x0CB3] = (1,) # GL_PIXEL_MAP_I_TO_G_SIZE
_m[0x0CB0] = (1,) # GL_PIXEL_MAP_I_TO_I_SIZE
_m[0x0CB2] = (1,) # GL_PIXEL_MAP_I_TO_R_SIZE
_m[0x0CB6] = (1,) # GL_PIXEL_MAP_R_TO_R_SIZE
_m[0x0CB1] = (1,) # GL_PIXEL_MAP_S_TO_S_SIZE
_m[0x88ED] = (1,) # GL_PIXEL_PACK_BUFFER_BINDING_ARB
_m[0x88ED] = (1,) # GL_PIXEL_PACK_BUFFER_BINDING_EXT
_m[0x8336] = (1,) # GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT
_m[0x88EF] = (1,) # GL_PIXEL_UNPACK_BUFFER_BINDING_ARB
_m[0x88EF] = (1,) # GL_PIXEL_UNPACK_BUFFER_BINDING_EXT
_m[0x87F3] = (1,) # GL_PN_TRIANGLES_NORMAL_MODE_ATI
_m[0x87F2] = (1,) # GL_PN_TRIANGLES_POINT_MODE_ATI
_m[0x87F4] = (1,) # GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI
_m[0x8129] = (3,) # GL_POINT_DISTANCE_ATTENUATION
_m[0x8129] = (3,) # GL_POINT_DISTANCE_ATTENUATION_ARB
_m[0x8128] = (1,) # GL_POINT_FADE_THRESHOLD_SIZE
_m[0x8128] = (1,) # GL_POINT_FADE_THRESHOLD_SIZE_ARB
_m[0x0B11] = (1,) # GL_POINT_SIZE
_m[0x0B13] = (1,) # GL_POINT_SIZE_GRANULARITY
_m[0x8127] = (1,) # GL_POINT_SIZE_MAX
_m[0x8127] = (1,) # GL_POINT_SIZE_MAX_ARB
_m[0x8126] = (1,) # GL_POINT_SIZE_MIN
_m[0x8126] = (1,) # GL_POINT_SIZE_MIN_ARB
_m[0x0B12] = (2,) # GL_POINT_SIZE_RANGE
_m[0x0B10] = (1,) # GL_POINT_SMOOTH
_m[0x0C51] = (1,) # GL_POINT_SMOOTH_HINT
_m[0x8861] = (1,) # GL_POINT_SPRITE
_m[0x8861] = (1,) # GL_POINT_SPRITE_ARB
_m[0x8CA0] = (1,) # GL_POINT_SPRITE_COORD_ORIGIN
_m[0x8861] = (1,) # GL_POINT_SPRITE_NV
_m[0x8863] = (1,) # GL_POINT_SPRITE_R_MODE_NV
_m[0x0B40] = (2,) # GL_POLYGON_MODE
_m[0x8039] = (1,) # GL_POLYGON_OFFSET_BIAS_EXT
_m[0x8037] = (1,) # GL_POLYGON_OFFSET_EXT
_m[0x8038] = (1,) # GL_POLYGON_OFFSET_FACTOR
_m[0x8038] = (1,) # GL_POLYGON_OFFSET_FACTOR_EXT
_m[0x8037] = (1,) # GL_POLYGON_OFFSET_FILL
_m[0x2A02] = (1,) # GL_POLYGON_OFFSET_LINE
_m[0x2A01] = (1,) # GL_POLYGON_OFFSET_POINT
_m[0x2A00] = (1,) # GL_POLYGON_OFFSET_UNITS
_m[0x0B41] = (1,) # GL_POLYGON_SMOOTH
_m[0x0C53] = (1,) # GL_POLYGON_SMOOTH_HINT
_m[0x0B42] = (1,) # GL_POLYGON_STIPPLE
_m[0x80BB] = (1,) # GL_POST_COLOR_MATRIX_ALPHA_BIAS
_m[0x80BB] = (1,) # GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI
_m[0x80B7] = (1,) # GL_POST_COLOR_MATRIX_ALPHA_SCALE
_m[0x80B7] = (1,) # GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI
_m[0x80BA] = (1,) # GL_POST_COLOR_MATRIX_BLUE_BIAS
_m[0x80BA] = (1,) # GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI
_m[0x80B6] = (1,) # GL_POST_COLOR_MATRIX_BLUE_SCALE
_m[0x80B6] = (1,) # GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI
_m[0x80D2] = (1,) # GL_POST_COLOR_MATRIX_COLOR_TABLE
_m[0x80D2] = (1,) # GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI
_m[0x80B9] = (1,) # GL_POST_COLOR_MATRIX_GREEN_BIAS
_m[0x80B9] = (1,) # GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI
_m[0x80B5] = (1,) # GL_POST_COLOR_MATRIX_GREEN_SCALE
_m[0x80B5] = (1,) # GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI
_m[0x80B8] = (1,) # GL_POST_COLOR_MATRIX_RED_BIAS
_m[0x80B8] = (1,) # GL_POST_COLOR_MATRIX_RED_BIAS_SGI
_m[0x80B4] = (1,) # GL_POST_COLOR_MATRIX_RED_SCALE
_m[0x80B4] = (1,) # GL_POST_COLOR_MATRIX_RED_SCALE_SGI
_m[0x8023] = (1,) # GL_POST_CONVOLUTION_ALPHA_BIAS
_m[0x8023] = (1,) # GL_POST_CONVOLUTION_ALPHA_BIAS_EXT
_m[0x801F] = (1,) # GL_POST_CONVOLUTION_ALPHA_SCALE
_m[0x801F] = (1,) # GL_POST_CONVOLUTION_ALPHA_SCALE_EXT
_m[0x8022] = (1,) # GL_POST_CONVOLUTION_BLUE_BIAS
_m[0x8022] = (1,) # GL_POST_CONVOLUTION_BLUE_BIAS_EXT
_m[0x801E] = (1,) # GL_POST_CONVOLUTION_BLUE_SCALE
_m[0x801E] = (1,) # GL_POST_CONVOLUTION_BLUE_SCALE_EXT
_m[0x80D1] = (1,) # GL_POST_CONVOLUTION_COLOR_TABLE
_m[0x80D1] = (1,) # GL_POST_CONVOLUTION_COLOR_TABLE_SGI
_m[0x8021] = (1,) # GL_POST_CONVOLUTION_GREEN_BIAS
_m[0x8021] = (1,) # GL_POST_CONVOLUTION_GREEN_BIAS_EXT
_m[0x801D] = (1,) # GL_POST_CONVOLUTION_GREEN_SCALE
_m[0x801D] = (1,) # GL_POST_CONVOLUTION_GREEN_SCALE_EXT
_m[0x8020] = (1,) # GL_POST_CONVOLUTION_RED_BIAS
_m[0x8020] = (1,) # GL_POST_CONVOLUTION_RED_BIAS_EXT
_m[0x801C] = (1,) # GL_POST_CONVOLUTION_RED_SCALE
_m[0x801C] = (1,) # GL_POST_CONVOLUTION_RED_SCALE_EXT
_m[0x817B] = (1,) # GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX
_m[0x817C] = (1,) # GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX
_m[0x8559] = (1,) # GL_PRIMITIVE_RESTART_INDEX_NV
_m[0x8558] = (1,) # GL_PRIMITIVE_RESTART_NV
_m[0x864B] = (1,) # GL_PROGRAM_ERROR_POSITION_ARB
_m[0x864B] = (1,) # GL_PROGRAM_ERROR_POSITION_NV
_m[0x8642] = (1,) # GL_PROGRAM_POINT_SIZE_ARB
_m[0x8642] = (1,) # GL_PROGRAM_POINT_SIZE_EXT
_m[0x0BA7] = (4, 4) # GL_PROJECTION_MATRIX
_m[0x0BA4] = (1,) # GL_PROJECTION_STACK_DEPTH
_m[0x8E4F] = (1,) # GL_PROVOKING_VERTEX
_m[0x8E4F] = (1,) # GL_PROVOKING_VERTEX_EXT
_m[0x8E4C] = (1,) # GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION
_m[0x8E4C] = (1,) # GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION_EXT
_m[0x8864] = (1,) # GL_QUERY_COUNTER_BITS
_m[0x8864] = (1,) # GL_QUERY_COUNTER_BITS_ARB
_m[0x19262] = (1,) # GL_RASTER_POSITION_UNCLIPPED_IBM
_m[0x0C02] = (1,) # GL_READ_BUFFER
_m[0x887B] = (1,) # GL_READ_PIXEL_DATA_RANGE_LENGTH_NV
_m[0x0D15] = (1,) # GL_RED_BIAS
_m[0x0D52] = (1,) # GL_RED_BITS
_m[0x8564] = (1,) # GL_RED_MAX_CLAMP_INGR
_m[0x8560] = (1,) # GL_RED_MIN_CLAMP_INGR
_m[0x0D14] = (1,) # GL_RED_SCALE
_m[0x8522] = (1,) # GL_REGISTER_COMBINERS_NV
_m[0x8CA7] = (1,) # GL_RENDERBUFFER_BINDING
_m[0x8CA7] = (1,) # GL_RENDERBUFFER_BINDING_EXT
_m[0x87FD] = (1,) # GL_RENDERBUFFER_FREE_MEMORY_ATI
_m[0x0C40] = (1,) # GL_RENDER_MODE
_m[0x85C2] = (1,) # GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN
_m[0x85C1] = (1,) # GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN
_m[0x81D8] = (1,) # GL_REPLACEMENT_CODE_SUN
_m[0x803A] = (1,) # GL_RESCALE_NORMAL
_m[0x8820] = (1,) # GL_RGBA_FLOAT_MODE_ARB
_m[0x0C31] = (1,) # GL_RGBA_MODE
_m[0x80A9] = (1,) # GL_SAMPLES
_m[0x86B4] = (1,) # GL_SAMPLES_3DFX
_m[0x80A9] = (1,) # GL_SAMPLES_ARB
_m[0x80A9] = (1,) # GL_SAMPLES_SGIS
_m[0x809E] = (1,) # GL_SAMPLE_ALPHA_TO_COVERAGE
_m[0x809E] = (1,) # GL_SAMPLE_ALPHA_TO_COVERAGE_ARB
_m[0x809E] = (1,) # GL_SAMPLE_ALPHA_TO_MASK_EXT
_m[0x809E] = (1,) # GL_SAMPLE_ALPHA_TO_MASK_SGIS
_m[0x809F] = (1,) # GL_SAMPLE_ALPHA_TO_ONE
_m[0x809F] = (1,) # GL_SAMPLE_ALPHA_TO_ONE_ARB
_m[0x809F] = (1,) # GL_SAMPLE_ALPHA_TO_ONE_EXT
_m[0x809F] = (1,) # GL_SAMPLE_ALPHA_TO_ONE_SGIS
_m[0x80A8] = (1,) # GL_SAMPLE_BUFFERS
_m[0x86B3] = (1,) # GL_SAMPLE_BUFFERS_3DFX
_m[0x80A8] = (1,) # GL_SAMPLE_BUFFERS_ARB
_m[0x80A8] = (1,) # GL_SAMPLE_BUFFERS_SGIS
_m[0x80A0] = (1,) # GL_SAMPLE_COVERAGE
_m[0x80A0] = (1,) # GL_SAMPLE_COVERAGE_ARB
_m[0x80AB] = (1,) # GL_SAMPLE_COVERAGE_INVERT
_m[0x80AB] = (1,) # GL_SAMPLE_COVERAGE_INVERT_ARB
_m[0x80AA] = (1,) # GL_SAMPLE_COVERAGE_VALUE
_m[0x80AA] = (1,) # GL_SAMPLE_COVERAGE_VALUE_ARB
_m[0x80A0] = (1,) # GL_SAMPLE_MASK_EXT
_m[0x80AB] = (1,) # GL_SAMPLE_MASK_INVERT_SGIS
_m[0x80A0] = (1,) # GL_SAMPLE_MASK_SGIS
_m[0x80AA] = (1,) # GL_SAMPLE_MASK_VALUE_SGIS
_m[0x80AC] = (1,) # GL_SAMPLE_PATTERN_SGIS
_m[0x0C10] = (4,) # GL_SCISSOR_BOX
_m[0x0C11] = (1,) # GL_SCISSOR_TEST
_m[0x845E] = (1,) # GL_SECONDARY_COLOR_ARRAY
_m[0x889C] = (1,) # GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING
_m[0x889C] = (1,) # GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB
_m[0x8F31] = (1,) # GL_SECONDARY_COLOR_ARRAY_LENGTH_NV
_m[0x845A] = (1,) # GL_SECONDARY_COLOR_ARRAY_SIZE
_m[0x845A] = (1,) # GL_SECONDARY_COLOR_ARRAY_SIZE_EXT
_m[0x845C] = (1,) # GL_SECONDARY_COLOR_ARRAY_STRIDE
_m[0x845C] = (1,) # GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT
_m[0x845B] = (1,) # GL_SECONDARY_COLOR_ARRAY_TYPE
_m[0x845B] = (1,) # GL_SECONDARY_COLOR_ARRAY_TYPE_EXT
_m[0x0DF4] = (1,) # GL_SELECTION_BUFFER_SIZE
_m[0x8012] = (1,) # GL_SEPARABLE_2D
_m[0x8012] = (1,) # GL_SEPARABLE_2D_EXT
_m[0x0B54] = (1,) # GL_SHADE_MODEL
_m[0x0B23] = (1,) # GL_SMOOTH_LINE_WIDTH_GRANULARITY
_m[0x0B22] = (2,) # GL_SMOOTH_LINE_WIDTH_RANGE
_m[0x0B13] = (1,) # GL_SMOOTH_POINT_SIZE_GRANULARITY
_m[0x0B12] = (2,) # GL_SMOOTH_POINT_SIZE_RANGE
_m[0x8801] = (1,) # GL_STENCIL_BACK_FAIL
_m[0x8800] = (1,) # GL_STENCIL_BACK_FUNC
_m[0x8802] = (1,) # GL_STENCIL_BACK_PASS_DEPTH_FAIL
_m[0x8803] = (1,) # GL_STENCIL_BACK_PASS_DEPTH_PASS
_m[0x8CA3] = (1,) # GL_STENCIL_BACK_REF
_m[0x8CA4] = (1,) # GL_STENCIL_BACK_VALUE_MASK
_m[0x8CA5] = (1,) # GL_STENCIL_BACK_WRITEMASK
_m[0x0D57] = (1,) # GL_STENCIL_BITS
_m[0x0B91] = (1,) # GL_STENCIL_CLEAR_VALUE
_m[0x0B94] = (1,) # GL_STENCIL_FAIL
_m[0x0B92] = (1,) # GL_STENCIL_FUNC
_m[0x0B95] = (1,) # GL_STENCIL_PASS_DEPTH_FAIL
_m[0x0B96] = (1,) # GL_STENCIL_PASS_DEPTH_PASS
_m[0x0B97] = (1,) # GL_STENCIL_REF
_m[0x0B90] = (1,) # GL_STENCIL_TEST
_m[0x8910] = (1,) # GL_STENCIL_TEST_TWO_SIDE_EXT
_m[0x0B93] = (1,) # GL_STENCIL_VALUE_MASK
_m[0x0B98] = (1,) # GL_STENCIL_WRITEMASK
_m[0x0C33] = (1,) # GL_STEREO
_m[0x0D50] = (1,) # GL_SUBPIXEL_BITS
_m[0x8439] = (1,) # GL_TANGENT_ARRAY_EXT
_m[0x843F] = (1,) # GL_TANGENT_ARRAY_STRIDE_EXT
_m[0x843E] = (1,) # GL_TANGENT_ARRAY_TYPE_EXT
_m[0x0DE0] = (1,) # GL_TEXTURE_1D
_m[0x8068] = (1,) # GL_TEXTURE_1D_BINDING_EXT
_m[0x0DE1] = (1,) # GL_TEXTURE_2D
_m[0x8069] = (1,) # GL_TEXTURE_2D_BINDING_EXT
_m[0x806F] = (1,) # GL_TEXTURE_3D
_m[0x806A] = (1,) # GL_TEXTURE_3D_BINDING_EXT
_m[0x806F] = (1,) # GL_TEXTURE_3D_EXT
_m[0x8068] = (1,) # GL_TEXTURE_BINDING_1D
_m[0x8C1C] = (1,) # GL_TEXTURE_BINDING_1D_ARRAY_EXT
_m[0x8069] = (1,) # GL_TEXTURE_BINDING_2D
_m[0x8C1D] = (1,) # GL_TEXTURE_BINDING_2D_ARRAY_EXT
_m[0x9104] = (1,) # GL_TEXTURE_BINDING_2D_MULTISAMPLE
_m[0x9105] = (1,) # GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY
_m[0x806A] = (1,) # GL_TEXTURE_BINDING_3D
_m[0x8C2C] = (1,) # GL_TEXTURE_BINDING_BUFFER_ARB
_m[0x8C2C] = (1,) # GL_TEXTURE_BINDING_BUFFER_EXT
_m[0x8514] = (1,) # GL_TEXTURE_BINDING_CUBE_MAP
_m[0x8514] = (1,) # GL_TEXTURE_BINDING_CUBE_MAP_ARB
_m[0x84F6] = (1,) # GL_TEXTURE_BINDING_RECTANGLE
_m[0x84F6] = (1,) # GL_TEXTURE_BINDING_RECTANGLE_ARB
_m[0x84F6] = (1,) # GL_TEXTURE_BINDING_RECTANGLE_NV
_m[0x8E53] = (1,) # GL_TEXTURE_BINDING_RENDERBUFFER_NV
_m[0x8C2A] = (1,) # GL_TEXTURE_BUFFER_ARB
_m[0x8C2D] = (1,) # GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB
_m[0x8C2D] = (1,) # GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT
_m[0x8C2A] = (1,) # GL_TEXTURE_BUFFER_EXT
_m[0x8C2E] = (1,) # GL_TEXTURE_BUFFER_FORMAT_ARB
_m[0x8C2E] = (1,) # GL_TEXTURE_BUFFER_FORMAT_EXT
_m[0x80BC] = (1,) # GL_TEXTURE_COLOR_TABLE_SGI
_m[0x84EF] = (1,) # GL_TEXTURE_COMPRESSION_HINT
_m[0x84EF] = (1,) # GL_TEXTURE_COMPRESSION_HINT_ARB
_m[0x8078] = (1,) # GL_TEXTURE_COORD_ARRAY
_m[0x889A] = (1,) # GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING
_m[0x889A] = (1,) # GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB
_m[0x808B] = (1,) # GL_TEXTURE_COORD_ARRAY_COUNT_EXT
_m[0x8078] = (1,) # GL_TEXTURE_COORD_ARRAY_EXT
_m[0x8088] = (1,) # GL_TEXTURE_COORD_ARRAY_SIZE
_m[0x8088] = (1,) # GL_TEXTURE_COORD_ARRAY_SIZE_EXT
_m[0x808A] = (1,) # GL_TEXTURE_COORD_ARRAY_STRIDE
_m[0x808A] = (1,) # GL_TEXTURE_COORD_ARRAY_STRIDE_EXT
_m[0x8089] = (1,) # GL_TEXTURE_COORD_ARRAY_TYPE
_m[0x8089] = (1,) # GL_TEXTURE_COORD_ARRAY_TYPE_EXT
_m[0x8513] = (1,) # GL_TEXTURE_CUBE_MAP
_m[0x8513] = (1,) # GL_TEXTURE_CUBE_MAP_ARB
_m[0x87FC] = (1,) # GL_TEXTURE_FREE_MEMORY_ATI
_m[0x0C63] = (1,) # GL_TEXTURE_GEN_Q
_m[0x0C62] = (1,) # GL_TEXTURE_GEN_R
_m[0x0C60] = (1,) # GL_TEXTURE_GEN_S
_m[0x0C61] = (1,) # GL_TEXTURE_GEN_T
_m[0x0BA8] = (4, 4) # GL_TEXTURE_MATRIX
_m[0x84F5] = (1,) # GL_TEXTURE_RECTANGLE_ARB
_m[0x84F5] = (1,) # GL_TEXTURE_RECTANGLE_NV
_m[0x8E54] = (1,) # GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV
_m[0x86DE] = (1,) # GL_TEXTURE_SHADER_NV
_m[0x0BA5] = (1,) # GL_TEXTURE_STACK_DEPTH
_m[0x8200] = (1,) # GL_TEXT_FRAGMENT_SHADER_ATI
_m[0x8E25] = (1,) # GL_TRANSFORM_FEEDBACK_BINDING_NV
_m[0x8E24] = (1,) # GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV
_m[0x8C7F] = (1,) # GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV
_m[0x8E23] = (1,) # GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV
_m[0x84E6] = (4,4) # GL_TRANSPOSE_COLOR_MATRIX
_m[0x84E6] = (4,4) # GL_TRANSPOSE_COLOR_MATRIX_ARB
_m[0x88B7] = (4, 4) # GL_TRANSPOSE_CURRENT_MATRIX_ARB
_m[0x84E3] = (4,4) # GL_TRANSPOSE_MODELVIEW_MATRIX
_m[0x84E3] = (4,4) # GL_TRANSPOSE_MODELVIEW_MATRIX_ARB
_m[0x84E4] = (4,4) # GL_TRANSPOSE_PROJECTION_MATRIX
_m[0x84E4] = (4,4) # GL_TRANSPOSE_PROJECTION_MATRIX_ARB
_m[0x84E5] = (4,4) # GL_TRANSPOSE_TEXTURE_MATRIX
_m[0x84E5] = (4,4) # GL_TRANSPOSE_TEXTURE_MATRIX_ARB
_m[0x8A28] = (1,) # GL_UNIFORM_BUFFER_BINDING
_m[0x8DEF] = (1,) # GL_UNIFORM_BUFFER_BINDING_EXT
_m[0x8A34] = (1,) # GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT
_m[0x0CF5] = (1,) # GL_UNPACK_ALIGNMENT
_m[0x85B2] = (1,) # GL_UNPACK_CLIENT_STORAGE_APPLE
_m[0x800F] = (1,) # GL_UNPACK_CMYK_HINT_EXT
_m[0x806E] = (1,) # GL_UNPACK_IMAGE_HEIGHT
_m[0x806E] = (1,) # GL_UNPACK_IMAGE_HEIGHT_EXT
_m[0x0CF1] = (1,) # GL_UNPACK_LSB_FIRST
_m[0x8985] = (1,) # GL_UNPACK_RESAMPLE_OML
_m[0x842D] = (1,) # GL_UNPACK_RESAMPLE_SGIX
_m[0x0CF2] = (1,) # GL_UNPACK_ROW_LENGTH
_m[0x806D] = (1,) # GL_UNPACK_SKIP_IMAGES
_m[0x806D] = (1,) # GL_UNPACK_SKIP_IMAGES_EXT
_m[0x0CF4] = (1,) # GL_UNPACK_SKIP_PIXELS
_m[0x0CF3] = (1,) # GL_UNPACK_SKIP_ROWS
_m[0x0CF0] = (1,) # GL_UNPACK_SWAP_BYTES
_m[0x87FB] = (1,) # GL_VBO_FREE_MEMORY_ATI
_m[0x8074] = (1,) # GL_VERTEX_ARRAY
_m[0x85B5] = (1,) # GL_VERTEX_ARRAY_BINDING
_m[0x85B5] = (1,) # GL_VERTEX_ARRAY_BINDING_APPLE
_m[0x8896] = (1,) # GL_VERTEX_ARRAY_BUFFER_BINDING
_m[0x8896] = (1,) # GL_VERTEX_ARRAY_BUFFER_BINDING_ARB
_m[0x807D] = (1,) # GL_VERTEX_ARRAY_COUNT_EXT
_m[0x8074] = (1,) # GL_VERTEX_ARRAY_EXT
_m[0x8F2B] = (1,) # GL_VERTEX_ARRAY_LENGTH_NV
_m[0x851E] = (1,) # GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE
_m[0x851E] = (1,) # GL_VERTEX_ARRAY_RANGE_LENGTH_NV
_m[0x851F] = (1,) # GL_VERTEX_ARRAY_RANGE_VALID_NV
_m[0x807A] = (1,) # GL_VERTEX_ARRAY_SIZE
_m[0x807A] = (1,) # GL_VERTEX_ARRAY_SIZE_EXT
_m[0x807C] = (1,) # GL_VERTEX_ARRAY_STRIDE
_m[0x807C] = (1,) # GL_VERTEX_ARRAY_STRIDE_EXT
_m[0x807B] = (1,) # GL_VERTEX_ARRAY_TYPE
_m[0x807B] = (1,) # GL_VERTEX_ARRAY_TYPE_EXT
_m[0x8650] = (1,) # GL_VERTEX_ATTRIB_ARRAY0_NV
_m[0x865A] = (1,) # GL_VERTEX_ATTRIB_ARRAY10_NV
_m[0x865B] = (1,) # GL_VERTEX_ATTRIB_ARRAY11_NV
_m[0x865C] = (1,) # GL_VERTEX_ATTRIB_ARRAY12_NV
_m[0x865D] = (1,) # GL_VERTEX_ATTRIB_ARRAY13_NV
_m[0x865E] = (1,) # GL_VERTEX_ATTRIB_ARRAY14_NV
_m[0x865F] = (1,) # GL_VERTEX_ATTRIB_ARRAY15_NV
_m[0x8651] = (1,) # GL_VERTEX_ATTRIB_ARRAY1_NV
_m[0x8652] = (1,) # GL_VERTEX_ATTRIB_ARRAY2_NV
_m[0x8653] = (1,) # GL_VERTEX_ATTRIB_ARRAY3_NV
_m[0x8654] = (1,) # GL_VERTEX_ATTRIB_ARRAY4_NV
_m[0x8655] = (1,) # GL_VERTEX_ATTRIB_ARRAY5_NV
_m[0x8656] = (1,) # GL_VERTEX_ATTRIB_ARRAY6_NV
_m[0x8657] = (1,) # GL_VERTEX_ATTRIB_ARRAY7_NV
_m[0x8658] = (1,) # GL_VERTEX_ATTRIB_ARRAY8_NV
_m[0x8659] = (1,) # GL_VERTEX_ATTRIB_ARRAY9_NV
_m[0x86A7] = (1,) # GL_VERTEX_BLEND_ARB
_m[0x8620] = (1,) # GL_VERTEX_PROGRAM_ARB
_m[0x864A] = (1,) # GL_VERTEX_PROGRAM_BINDING_NV
_m[0x8620] = (1,) # GL_VERTEX_PROGRAM_NV
_m[0x8DA2] = (1,) # GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV
_m[0x8642] = (1,) # GL_VERTEX_PROGRAM_POINT_SIZE
_m[0x8642] = (1,) # GL_VERTEX_PROGRAM_POINT_SIZE_ARB
_m[0x8642] = (1,) # GL_VERTEX_PROGRAM_POINT_SIZE_NV
_m[0x8643] = (1,) # GL_VERTEX_PROGRAM_TWO_SIDE
_m[0x8643] = (1,) # GL_VERTEX_PROGRAM_TWO_SIDE_ARB
_m[0x8643] = (1,) # GL_VERTEX_PROGRAM_TWO_SIDE_NV
_m[0x8781] = (1,) # GL_VERTEX_SHADER_BINDING_EXT
_m[0x8780] = (1,) # GL_VERTEX_SHADER_EXT
_m[0x87D4] = (1,) # GL_VERTEX_SHADER_OPTIMIZED_EXT
_m[0x850D] = (1,) # GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT
_m[0x850F] = (1,) # GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT
_m[0x850E] = (1,) # GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT
_m[0x8719] = (1,) # GL_VIBRANCE_BIAS_NV
_m[0x8713] = (1,) # GL_VIBRANCE_SCALE_NV
_m[0x9021] = (1,) # GL_VIDEO_BUFFER_BINDING_NV
_m[0x0BA2] = (4,) # GL_VIEWPORT
_m[0x889E] = (1,) # GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB
_m[0x86AB] = (1,) # GL_WEIGHT_ARRAY_SIZE_ARB
_m[0x86AA] = (1,) # GL_WEIGHT_ARRAY_STRIDE_ARB
_m[0x86A9] = (1,) # GL_WEIGHT_ARRAY_TYPE_ARB
_m[0x86A6] = (1,) # GL_WEIGHT_SUM_UNITY_ARB
_m[0x887A] = (1,) # GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV
_m[0x0D16] = (1,) # GL_ZOOM_X
_m[0x0D17] = (1,) # GL_ZOOM_Y
